# -*- coding: UTF-8 -*-
# @Time     : 2/4/21 3:58 PM
# @Author   : Jackie
# @File     : utilFtp.py
import traceback
from ftplib import FTP_TLS
from .logger import logger


class UtilFtp:
    def __init__(self, config):
        self.config = config
        self.ftp = FTP_TLS(self.config['host'])
        if config['debug_level'] in ('0', '1', '2', 0, 1, 2):
            self.ftp.set_debuglevel(int(config['debug_level']))
        self.login()

    def login(self):
        logger.info('Start Login ftp[%s:%s]' % (self.config['host'], self.config['port']))
        self.ftp.auth()
        username, password = self.config['username'], self.config['password']
        self.ftp.login(user=username, passwd=password)
        self.ftp.makepasv()
        self.ftp.sendcmd('pbsz 0')
        self.ftp.set_pasv(True)
        self.ftp.prot_p()
        logger.info('Ftp Login success[%s]' % (self.config['username']))

    def upload(self, file_path, ftp_tar_path, **kwargs):
        logger.info('Start upload file[%s], tar[%s].' % (file_path, ftp_tar_path))
        with open(file_path, 'rb') as f_up:
            self.ftp.storbinary('STOR ' + ftp_tar_path, f_up, **kwargs)
        logger.info('Upload file[%s] success.' % ftp_tar_path)

    def download(self, ftp_file_path, tar_path, **kwargs):
        logger.info('Start download file[%s], tar[%s].' % (ftp_file_path, tar_path))
        with open(ftp_file_path, 'wb') as f_down:
            self.ftp.retrbinary('RETR %s' % tar_path, f_down.write, **kwargs)
        logger.info('Download file[%s] success.' % tar_path)

    def trans_file(self, source_path, tar_path, action, **kwargs):
        try:
            if action == 'push':
                self.upload(source_path, tar_path, **kwargs)
            elif action == 'pull':
                self.download(source_path, tar_path, **kwargs)
            else:
                logger.error('FTP un support action[%s].' % action)
                raise RuntimeError
        except Exception as e:
            logger.error('FTP [%s] file fail.Throw exception[%s]' % (action, str(e)))
            logger.error('traceback.format_exc():\n%s' % traceback.format_exc())

    def quit(self):
        self.ftp.quit()
        logger.info('Quit ftp success.')



# -*- coding: UTF-8 -*-
# @Time     : 5/21/21 4:57 PM
# @Author   : Jackie
# @File     : handlerEmail.py

from email.header import Header
from email.mime.text import MIMEText
import smtplib

from .logger import logger


def get_email_server(email_config):
    # email 相关配置
    from_addr = email_config['username']
    password_email = email_config['password']
    smtp_server = email_config['server.host']
    smtp_port = int(email_config['server.port'])
    email_timeout = email_config['timeout']

    if type(email_timeout) not in (int, float, str):
        email_timeout = 60
    else:
        email_timeout = float(email_timeout)

    server = smtplib.SMTP(smtp_server, smtp_port, timeout=email_timeout)
    # server.connect(host=smtp_server, port=smtp_port)
    server.starttls()
    server.login(from_addr, password_email)
    return server


class HandlerEmail:
    def __init__(self, email_config):
        self.email_config = email_config
        self.to_addr_list = email_config['receivers']
        self.subject = email_config['title']
        self.body = email_config['content']

    def send_email(self, subtype='html'):
        msg = MIMEText('%s' % self.body, _subtype=subtype, _charset='utf-8')

        msg['From'] = self.email_config['username']
        msg['To'] = ','.join(self.to_addr_list)
        msg['Subject'] = Header('%s' % self.subject, 'utf-8').encode()

        server = get_email_server(self.email_config)

        try:
            server.sendmail(self.email_config['username'], self.to_addr_list, msg.as_string())
            logger.info("send email success. to[%s], subject[%s]" % (str(self.to_addr_list), self.subject))
            return True
        finally:
            server.quit()



# -*- coding: UTF-8 -*-
# @Time     : 2020/8/27 11:00
# @Author   : Jackie
# @File     : handlerMysql.py

import pymysql

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from .read_config import apollo_reader
from .logger import logger


def singleton(cls):
    instances = {}

    def wrapper(*args, **kwargs):
        # 通过 对象+参数 共同判断单例的唯一性
        _obj_key = '-'.join([str(cls), str(sorted(args)), str(sorted(kwargs.items()))])
        if _obj_key not in instances:
            instances[_obj_key] = cls(*args, **kwargs)
        return instances.get(_obj_key)

    return wrapper


@singleton
class MyDB:
    def __init__(self, database='default'):
        apollo_configurations = apollo_reader.get_configurations()
        mysql_hosts = [key for key in apollo_configurations.keys() if key.startswith('mysql') and key.endswith('host')]
        databases = [database.split('.')[1] for database in mysql_hosts]

        self.database = database
        if self.database not in databases:
            self.database = 'default'

        db_config = {
            'host': apollo_reader.get_value('mysql.%s.host' % str(self.database)),
            'user': apollo_reader.get_value('mysql.%s.user' % str(self.database)),
            'passwd': apollo_reader.get_value('mysql.%s.pwd' % str(self.database)),
            'port': int(apollo_reader.get_value('mysql.%s.port' % str(self.database))),
            'db': apollo_reader.get_value('mysql.%s.database' % str(self.database)),
            'autocommit': True,
        }

        self.db = pymysql.connect(**db_config)
        self.cursor = self.db.cursor()
        self.cursor_dict = self.db.cursor(cursor=pymysql.cursors.DictCursor)

    def execute_sql(self, sql, is_dict=False, is_commit=True):
        self.ping()
        if is_dict:
            self.cursor_dict.execute(sql)
            cursor = self.cursor_dict
        else:
            self.cursor.execute(sql)
            cursor = self.cursor
        if is_commit:
            self.db.commit()
        return cursor

    def get_all(self, is_dict=False):
        if is_dict:
            value = self.cursor_dict.fetchall()
        else:
            value = self.cursor.fetchall()
        return value

    def get_one(self):
        value = self.cursor.fetchone()
        if value is not None:
            try:
                return value[0]
            except IndexError:
                return None
        return value

    def commit(self):
        self.db.commit()

    def ping(self, reconnect=True):
        self.db.ping(reconnect)

    def close_db(self):
        self.db.close()
        logger.info("Database closed!")


class DBManagerORM:

    def __init__(self) -> None:
        self.db_cache = {}

    def init_db(self, db_name):
        db_url = 'mysql+pymysql://{username}:{password}@{host}:{port}/{database}?charset=utf8mb4&' \
                 'autocommit={autocommit}'
        configurations = apollo_reader.get_configurations()
        mysql_hosts = [key for key in configurations.keys() if key.startswith('mysql') and key.endswith('host')]
        databases = [database.split('.')[1] for database in mysql_hosts]

        if db_name not in databases:
            db_name = 'default'

        db_setting = {
            'host': apollo_reader.get_value(f'mysql.{db_name}.host'),
            'port': apollo_reader.get_value(f'mysql.{db_name}.port'),
            'username': apollo_reader.get_value(f'mysql.{db_name}.user'),
            'password': apollo_reader.get_value(f'mysql.{db_name}.pwd'),
            'database': apollo_reader.get_value(f'mysql.{db_name}.database'),
            'autocommit': 'true',
        }
        db_url = db_url.format(**db_setting)
        if db_name not in self.db_cache:
            engine = create_engine(db_url, echo=False, max_overflow=5)
            session = sessionmaker(bind=engine)()
            self.db_cache[db_name] = session
        return self.db_cache[db_name]

    def close_db(self):
        for _, db in self.db_cache.items():
            db.disconnect()
        self.db_cache = {}

    def get_db(self, db_name):
        if db_name in self.db_cache:
            return self.db_cache[db_name]
        return self.init_db(db_name)


class DBManager:
    def __init__(self) -> None:
        self.db_cache = {}

    def init_db(self, db_name) -> MyDB:
        if db_name not in self.db_cache:
            self.db_cache[db_name] = MyDB(database=db_name)
        return self.db_cache[db_name]

    def close_db(self):
        for _, db in self.db_cache.items():
            db.close_db()
        self.db_cache = {}

    def get_db(self, db_name):
        if db_name in self.db_cache:
            return self.db_cache[db_name]
        return self.init_db(db_name)


db_manager_orm = DBManagerORM()
db_manager = DBManager()
